/*
 * cmd_file.c - Commands to deal with files
 * $Fuller: myprograms/fread/cmd_file.c,v 1.4 2003/06/06 09:04:06 fullermd Exp $
 */

#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <fread.h>


/* Pull in globals */
extern FR_FILE **allfiles;
extern int numfiles, curfile;


/* Open a new file */
void
cmd_open(const char *file)
{
	FR_FILE **alltmp;
	FR_FILE *newfile;
	char *spc;

	if((newfile=fr_alloc_frfile())==NULL)
		return;

	if( (file==NULL) || (strlen(file)<1))
	{
		printf("Usage: open <filename>\n");
		return;
	}

	/* Try and give good errors */
	if(*file=='~')
	{
		printf("I'm stupid, I don't do ~-expansion yet.\n");
		return;
	}

	/* We want our own copy! */
	newfile->name = strdup(file);
	if(newfile->name==NULL)
	{
		perror("malloc()");
		fr_free_frfile(newfile);
		return;
	}

	/* Whack trailing spaces */
	spc = newfile->name;
	spc += strlen(spc);
	while(*--spc==' ')
		*spc='\0';

	/* fr_open() does all our error-checking */
	if(fr_open(newfile)<0)
		return;
	
	/*
	 * If we made it here, the file is open.
	 * Now, add it into our global structure.
	 */
	alltmp = realloc(allfiles, (sizeof(FR_FILE *) * (numfiles+1)));
	if(alltmp==NULL)
	{
		perror("malloc()");
		fr_close(newfile);
		fr_free_frfile(newfile);
		return;
	}
	allfiles = alltmp;
	allfiles[numfiles]=newfile;
	fr_setcurfile(numfiles);
	numfiles++;

	/* We're done */
	printf("Open of %s succeeded!\n", newfile->name);
	return;
}


/* Close an open file */
void
cmd_close(const char *filenum_s)
{
	FR_FILE **allnew;
	char *fname;
	int i,j;
	int filenum;

	/* Well, first off... */
	if(numfiles==0)
	{
		printf("Error: No files are open.\n");
		return;
	}

	/*
	 * Get what we're doing.
	 * If no arg, use curfile.
	 */ 
	if(*filenum_s=='\0')
		filenum = curfile;
	else
		filenum = atoi(filenum_s);
		if(filenum<1)
		{
			printf("Usage: close <num>\n");
			return;
		}

	/* Sanity checks */
	if(filenum>numfiles)
	{
		printf("Error: Only %d files are open.\n", numfiles);
		return;
	}
	if(allfiles[filenum-1]->desc<=0)
	{
		printf("Error: File #%d not open.\n", filenum);
		printf("This probably shouldn't be possible, so this may be\n");
		printf("a bug in fread...\n");
		return;
	}

	/* It's open.  Close it and eliminate it */
	fr_close(allfiles[filenum-1]);
	fname = strdup(allfiles[filenum-1]->name);
	fr_free_frfile(allfiles[filenum-1]);
	allnew = malloc(sizeof(FR_FILE *) * (numfiles-1));
	if(allnew==NULL)
	{
		perror("malloc()");
		return;
	}

	j=0;
	for(i=0 ; i < numfiles ; i++)
	{
		if(i!=(filenum-1))
			allnew[j++] = allfiles[i];
	}

	free(allfiles);
	allfiles=allnew;
	numfiles--;

	/*
	 * This needs some explanation.
	 * If curfile was after this one, leave it alone.
	 * If curfile was equal to this one, we want to shift it back one
	 * space, and it'll become the one "before" this one, as we want it
	 * to.
	 * If curfile was before this one, move it back one slot so it'll
	 * still point at the same one.
	 */ 
	if( (filenum-1)<=curfile )
		fr_setcurfile(curfile-1);

	/* That's it */
	if(fname==NULL)
		perror("malloc()");
	else
	{
		printf("Close of file %d (%s) succeeded!\n", filenum, fname);
		free(fname);
	}

	return;
}


/* List open files */
void
cmd_list(const char *dummy)
{
	int i;

	if(numfiles==0)
	{
		printf("No open files\n");
		return;
	}

	printf("Open files:\n");
	printf("%3s %-40s %6s %10s\n", "Num", "Filename", "Offset",
			"Bytes read");
	for(i=0 ; i<numfiles ; i++)
		printf("%3d %-40s %6li %10li\n", (i+1), allfiles[i]->name,
				(long)allfiles[i]->curoffset, (long)allfiles[i]->buflen);

	return;
}


/* Switch between files */
void
cmd_file(const char *filenum_s)
{
	int filenum;

	/* Well, first off... */
	if(numfiles==0)
	{
		printf("Error: No files are open.\n");
		return;
	}

	/* Get what we're doing */
	filenum = atoi(filenum_s);
	if(filenum<1)
	{
		printf("Usage: file <num>\n");
		return;
	}

	/* Sanity checks */
	if(filenum>numfiles)
	{
		printf("Error: Only %d files are open.\n", numfiles);
		return;
	}
	if(allfiles[filenum-1]->desc<=0)
	{
		printf("Error: File #%d not open.\n", filenum);
		printf("This probably shouldn't be possible, so this may be\n");
		printf("a bug in fread...\n");
		return;
	}


	/* OK, we're good.  Do the switch */
	fr_setcurfile(filenum-1);

	printf("Curfile changed!\n");
	return;
}


/* Seek in a file */
#define SEEK_USAGE \
{ \
	printf("Usage: seek <amount> [set]\n"); \
	printf("Seeks the current file by <amount> bytes.\n"); \
	printf("If 'set' is specified, seeks from the beginning of the file.\n"); \
	free(mycmd); \
	return; \
}
void
cmd_seek(const char *cmdstring)
{
	char *mycmd;
	char *findamt, *findwhence;
	off_t sdist=0, retval;
	int whence;

	mycmd = strdup(cmdstring);
	if(mycmd==NULL)
	{
		perror("malloc");
		return;
	}

	/* Split up our command */
	findamt = mycmd;
	if(*findamt=='\0')
		SEEK_USAGE
	if(*findamt=='-')
	{
		printf("Error: seek can only move positive amounts.\n");
		printf("(Yes, I should fix that.  Cope.)\n");
		return;
	}
	findwhence = strchr(findamt, ' ');
	if(findwhence==NULL)
		whence=SEEK_CUR;
	else
	{
		*findwhence = '\0';
		findwhence++;
		while( (*findwhence!='\0') && (*findwhence==' ') )
			findwhence++;
		if(strcmp(findwhence, "set")==0)
			whence=SEEK_SET;
		else if(strcmp(findwhence, "cur")==0)
			whence=SEEK_CUR;
		else
			SEEK_USAGE
	}

	/* Interpret our command */
	sdist = (off_t)atol(findamt);
	if(sdist==0)
	{
		printf("Error: Seek amount must be non-zero\n");
		SEEK_USAGE
	}
	free(mycmd);
	mycmd=NULL;


	/* OK, we seem to be good to go.  Gopher it */
	retval = lseek(allfiles[curfile]->desc, sdist, whence);
	if(retval<1)
	{
		perror("lseek");
		return;
	}

	/* Seek succeeded */
	allfiles[curfile]->curoffset = retval;
	printf("Seek'd file %d(%s) to offset %li\n", (curfile+1),
			allfiles[curfile]->name, (long)retval);

	return;
}
#undef SEEK_USAGE


/* Read from the current file */
void
cmd_read(const char *bytes_s)
{
	size_t bytes;
	ssize_t readret;
	off_t realoff;

	/* Well, first off... */
	if(curfile==-1)
	{
		printf("Error: No file selected.\n");
		return;
	}

	/* Get what we're doing */
	bytes = (size_t)atol(bytes_s);
	if(bytes <1)
	{
		printf("Usage: read <amount>\n");
		return;
	}

	if(bytes<=0)
	{
		printf("Doesn't make much sense to read %d bytes, does it?\n",
				bytes);
		return;
	}

	/* Save and check our current offset */
	realoff = lseek(allfiles[curfile]->desc, 0, SEEK_CUR);
	if(realoff != allfiles[curfile]->curoffset)
		printf("Warning: Current offset (%li) doesn't match what "
				"I'm expecting it to be (%li)...\n", (long)realoff,
				(long)allfiles[curfile]->curoffset);

	/* Give it a whirl */
	free(allfiles[curfile]->readbuf);
	allfiles[curfile]->readbuf = malloc(bytes+1);
	if(allfiles[curfile]->readbuf==NULL)
	{
		perror("malloc()");
		return;
	}
	allfiles[curfile]->bufsize = bytes;

	readret = read(allfiles[curfile]->desc, allfiles[curfile]->readbuf,
			bytes);
	
	if(readret<0)
	{
		perror("read()");
		return;
	}
	
	/* Give our messages and get the hell outta Dodge */
	if(readret!=bytes)
		printf("Warning: Only %li bytes read (%li requested)\n",
				(long)readret, (long)bytes);
	allfiles[curfile]->readbuf[readret] = '\0';
	allfiles[curfile]->buflen = readret;

	lseek(allfiles[curfile]->desc, realoff, SEEK_SET);
	allfiles[curfile]->curoffset = realoff;

	printf("%li bytes read into buffer\n", (long)readret);
	return;
}
