/*
 * cmd_proc.c - Command processing
 * $Fuller: myprograms/fread/cmd_proc.c,v 1.4 2003/06/06 10:13:26 fullermd Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fread.h>

struct cmd_list
{
	char		*name;
	void		(*func)(const char *);
};

struct cmd_list cmd_table[] =
{
	/*
	 * Order of this is important.
	 * If we have a "helpnowaitquit" command that really does 'quit', and
	 * we put is before 'help', then anytime you type 'help' it'll quit.
	 * We can also abbreviate, so "h" will run the first command that
	 * starts with "h", "he" will run the first "he", etc.
	 */ 
	{ "quit",       cmd_quit    },
	{ "help",       cmd_help    },
	{ "?",          cmd_help    },
	{ "open",       cmd_open    },
	{ "close",      cmd_close   },
	{ "list",       cmd_list    },
	{ "seek",       cmd_seek    },
	{ "read",       cmd_read    },
	{ "print",      cmd_print   },
	{ "file",       cmd_file    },
	{ "less",       cmd_less    },
	{ "version",    cmd_version },

	/* Aliases have a lower priority */
	{ "more",       cmd_less    },
	{ "page",       cmd_less    },

	/* End with null */
	{ NULL,			NULL },
};


/* Process a command */
void
cmd_proc(const char *cmd)
{
	int i, cmdlen;
	char *cmdcomp, *buf;
	const char *cmdshift;

	cmdcomp = strdup(cmd);
	if(cmdcomp==NULL)
	{
		perror("strdup()");
		return;
	}
	buf = strchr(cmdcomp, ' ');
	if(buf!=NULL)
		*buf='\0';
	cmdlen = strlen(cmdcomp);

	/* Loop through the table trying to find our command */
	for(i=0 ; cmd_table[i].name!=NULL ; i++)
	{
		if((strncmp(cmdcomp, cmd_table[i].name, cmdlen))==0)
		{
			/* Match */
			cmdshift=cmd;
			while((*cmdshift!=' ') && (*cmdshift!='\0'))
				cmdshift++;
			while((*cmdshift==' ') && (*cmdshift!='\0'))
				cmdshift++;
			(*cmd_table[i].func)(cmdshift);
			free(cmdcomp);
			return;
		}
	}

	/* If we get here, it's not found */
	printf("Error: Unknown command.  Try 'help'.\n");

	free(cmdcomp);
	return;
}
